# from mwauth import AuthType
# from mwauth.utils import AuthType
from .kong_auth import KongAuth
from .session_auth3 import SessionAuth
from .baseuser import BaseUser
from .utils import AuthType,auth,current_user


